/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include <math.h>

#include "EllipticHotspot.h"

EllipticHotspot::EllipticHotspot(int const &id, HWND hwnd)
: Hotspot(id, hwnd)
{}

EllipticHotspot::~EllipticHotspot()
{}

void EllipticHotspot::Move(Point const &pos)
{
	if(IsActive()) StopTimer();

	x_ = pos.x;
	y_ = pos.y;

	if(IsActive()) StartTimer();
}

void EllipticHotspot::MoveBy(Point const &dpos)
{
	if(IsActive()) StopTimer();

	x_ += dpos.x;
	y_ += dpos.y;

	if(IsActive()) StartTimer();
}

void EllipticHotspot::Resize(Point const &dim)
{
	if(IsActive()) StopTimer();

	radius_h_ = dim.x * dim.x;
	radius_v_ = dim.y * dim.y;

	if(IsActive()) StartTimer();
}

void EllipticHotspot::ResizeBy(Point const &ddim)
{
	if(IsActive()) StopTimer();

	int new_rad_h = static_cast<int>(sqrt(static_cast<float>(radius_h_)) + ddim.x);
	int new_rad_v = static_cast<int>(sqrt(static_cast<float>(radius_v_)) + ddim.y);
	radius_h_ = new_rad_h * new_rad_h;
	radius_v_ = new_rad_v * new_rad_v;

	if(IsActive()) StartTimer();
}
	
void EllipticHotspot::Reposition(Point const &pos, Point const &dim)
{
	bool actual_active = IsActive();
	if(actual_active)
	{
		StopTimer();
		SetActive(false);
	}

	Move(pos);
	Resize(dim);

	if(actual_active)
	{
		SetActive(true);
	}
}

void EllipticHotspot::RepositionBy(Point const &dpos, Point const &ddim)
{
	bool actual_active = IsActive();
	if(actual_active)
	{
		StopTimer();
		SetActive(false);
	}

	MoveBy(dpos);
	ResizeBy(ddim);

	if(actual_active)
	{
		SetActive(true);
	}
}

bool const EllipticHotspot::IsInside(Point const &mouse_pos)
{
	// formula: ( a^2 / r1^2 ) + ( b^2 / r2^2 ) = 1
	double xx = (mouse_pos.x - x_) * (mouse_pos.x - x_);
	double yy = (mouse_pos.y - y_) * (mouse_pos.y - y_);
	
	if(xx / radius_h_ + yy / radius_v_ <= 1)
		return true;
	else
		return false;

}