/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "HotspotManager.h"

HotspotManager::HotspotManager(HINSTANCE hInst)
: curr_id_(1)
{
	hotspotmanager = this;

	hwnd_ = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		"xHotspotManager", //CLASSNAME,
		0,		// Window Name
		WS_POPUP,		// Window style
		0, 0, 0, 0,		// zero-sized  
		0/*HWND_MESSAGE*/,	// message-only window
		0,				// hMenu
		hInst,		// hInstance
		0				// lpParam
	);
	if (!hwnd_)
		MessageBox(0, "xHotspot failed to create Hotspot window", "Error", MB_OK);
}

HotspotManager::~HotspotManager()
{
	for(std::map<std::string, Hotspot * >::iterator i = spots_.begin(); i != spots_.end(); ++i)
	{
		KillTimer(hwnd_, i->second->GetId());
		delete i->second;
	}
	DestroyWindow(hwnd_);
}

Hotspot *HotspotManager::AddSpot(std::string const &name, HotspotType const &type, HINSTANCE dll_inst)
{
	spotmap::iterator spot_iter = spots_.find(name);
	//LSLogPrintf(4, "xHotspot", "addspot %s", name.c_str());	
	if (spot_iter == spots_.end())
	{
		Hotspot *newspot;

		switch (type) {
			case Rectangular:
				//LSLogPrintf(4, "xHotspot", "add spot type=rect name=%s id=%d",name.c_str(), curr_id_);
				newspot = new RectHotspot(curr_id_, hwnd_);
				break;
			case Elliptic:
				//LSLogPrintf(4, "xHotspot", "add spot type=ellip name=%s",name.c_str());
				newspot = new EllipticHotspot(curr_id_, hwnd_);
				break;
			case NormalGroup:
				//LSLogPrintf(4, "xHotspot", "add spot type=group name=%s",name.c_str());
				newspot = new NormalGroupHotspot(curr_id_, hwnd_);
				break;
			case RealGroup:
				//LSLogPrintf(4, "xHotspot", "add spot type=group name=%s",name.c_str());
				newspot = new RealGroupHotspot(curr_id_, hwnd_);
				break;

		}

		spots_[name] = newspot;
		spot_id_.push_back(newspot);
		++curr_id_;

		return newspot;
	}
	else
		return spot_iter->second;
}

void HotspotManager::RemoveSpot(std::string const &name)
{
	spotmap::iterator spot_iter = spots_.find(name);
	if (spot_iter != spots_.end())
	{
		spot_id_[spot_iter->second->GetId() - 1] = 0;
		spots_.erase(spot_iter);
		delete spot_iter->second;
	}
}

Hotspot *HotspotManager::GetSpot(std::string const &name)
{
	/*for (spotmap::iterator i = spots_.begin(); i != spots_.end(); ++i)
	{
		LSLogPrintf(4, "xHotspot", "getspot %s =? %s", name.c_str(), (i->first).c_str());
	}*/
	spotmap::iterator spot_iter = spots_.find(name);
	if (spot_iter != spots_.end())
	{
		return spot_iter->second;
	}
	else
	{
		return 0;
	}
}

void HotspotManager::SetType(std::string const &name, HotspotType const &type)
{}

void HotspotManager::AddInitSpot(Hotspot *spot, bool state)
{
	init_[spot] = state;
}

void HotspotManager::InitSpots()
{
	for(spotinitmap::iterator it = init_.begin();
		it != init_.end(); ++it)
	{
		it->first->SetActive(it->second);
		//LSLogPrintf(4, "xHotspot", "initspots spotid=%d", it->first->GetId());
	}
	init_.clear();
}


LRESULT HotspotManager::WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//LSLogPrintf(4, "xHotspot", "windowproc message=%u wParam=%d", message, (int)wParam);

	if(message == WM_TIMER)
	{
		int pos = static_cast<int>(wParam);
		if(hotspotmanager->spot_id_.size() >= static_cast<UINT>(pos - 1))
		{
			Hotspot *spot = hotspotmanager->spot_id_[pos - 1];
			if (spot != 0)
			{
				MouseInOut result = spot->Check();
				return true;
			}
		}
		/*
		else
		{
			//LSLogPrintf(4, "xHotspot", "windowproc pos=%d not found", pos);
		}
		*/
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}