/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "RealGroupHotspot.h"

RealGroupHotspot::RealGroupHotspot(int const &id, HWND hwnd)
: HotspotGroup(id, hwnd)
{}

RealGroupHotspot::~RealGroupHotspot()
{}

void RealGroupHotspot::Move(Point const &pos)
{
	if(IsActive()) StopTimer();

	for(std::list<Hotspot* >::const_iterator subspot = subspots_.begin();
		subspot == subspots_.end();
		++subspot)
	{
		(*subspot)->Move(pos);
	}

	if(IsActive()) StartTimer();
}

void RealGroupHotspot::MoveBy(Point const &dpos)
{
	if(IsActive()) StopTimer();

	for(std::list<Hotspot* >::const_iterator subspot = subspots_.begin();
		subspot == subspots_.end();
		++subspot)
	{
		(*subspot)->MoveBy(dpos);
	}

	if(IsActive()) StartTimer();
}

void RealGroupHotspot::Resize(Point const &dim)
{}

void RealGroupHotspot::ResizeBy(Point const &ddim)
{}
	
void RealGroupHotspot::Reposition(Point const &pos, Point const &dim)
{}

void RealGroupHotspot::RepositionBy(Point const &dpos, Point const &ddim)
{}

bool const RealGroupHotspot::IsInside(Point const &mouse_pos)
{
	//LSLogPrintf(4, "xHotspot", "IsInside group %d",id_);
	if (subspots_.empty())
	{
		return false;
	}

	for(std::list<Hotspot* >::iterator subspot = subspots_.begin();
		subspot != subspots_.end();
		++subspot)
	{
		//LSLogPrintf(4, "xHotspot", "IsInside checking subspot %d", (*subspot)->GetId());
		if ((*subspot)->IsInside(mouse_pos) && (*subspot)->IsActive())
		{
			return true;
		}
	}

	return false;
}
