/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "RectHotspot.h"

RectHotspot::RectHotspot(int const &id, HWND hwnd)
: Hotspot(id, hwnd)
{
	//LSLogPrintf(4, "xHotspot", "created xhotspot id = %d", id_);
}

RectHotspot::~RectHotspot()
{}

void RectHotspot::Move(Point const &pos)
{
	if(IsActive()) StopTimer();

	x_ = pos.x;
	y_ = pos.y;

	if(IsActive()) StartTimer();
}

void RectHotspot::MoveBy(Point const &dpos)
{
	if(IsActive()) StopTimer();

	x_ += dpos.x;
	y_ += dpos.y;

	if(IsActive()) StartTimer();
}

void RectHotspot::Resize(Point const &dim)
{
	if(IsActive()) StopTimer();

	width_ = dim.x;
	height_ = dim.y;

	/*
	if(width_ < 1) width_ = 1;
	if(height_ < 1) height_ = 1;
	*/

	if(IsActive()) StartTimer();
}

void RectHotspot::ResizeBy(Point const &ddim)
{
	if(IsActive()) StopTimer();

	width_ += ddim.x;
	height_ += ddim.y;

	if(width_ < 1) width_ = 1;
	if(height_ < 1) height_ = 1;

	if(IsActive()) StartTimer();
}
	
void RectHotspot::Reposition(Point const &pos, Point const &dim)
{
	bool actual_active = IsActive();
	if(actual_active)
	{
		StopTimer();
		SetActive(false);
	}

	Move(pos);
	Resize(dim);

	if(actual_active)
	{
		SetActive(true);
	}
}

void RectHotspot::RepositionBy(Point const &dpos, Point const &ddim)
{
	bool actual_active = IsActive();
	if(actual_active)
	{
		StopTimer();
		SetActive(false);
	}

	MoveBy(dpos);
	ResizeBy(ddim);

	if(actual_active)
	{
		SetActive(true);
	}
}

bool const RectHotspot::IsInside(Point const &mouse_pos)
{
	if(
		(mouse_pos.x >= x_) && (mouse_pos.x < x_ + width_)
		&& (mouse_pos.y >= y_) && (mouse_pos.y < y_ + height_)
	)
		return true;
	else
		return false;

}
