/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "Util.h"

#include "stdafx.h"
#include <strsafe.h>

#include <lsapi/lsapi.h>

std::string const util::to_lowercase(std::string const &str)
{
	std::string result;
	for (std::string::const_iterator i = str.begin(); i != str.end(); ++i)
		result += tolower(*i);
	return result;
}

int const util::string_to_int(std::string const &value)
{
	return atoi(value.c_str());
}

int const util::parse_char_to_int(char const value[], int const &max_val)
{
	std::string value_str = value;
	return parse_string_to_int(value_str, max_val);
}

int const util::parse_string_to_int(std::string const &value, int const &max_val)
{
	bool real_negative = false;
	int start = 0;

	if(value[0] == '~')
	{
		real_negative = true;
		start = 1;
	}
	int result = ParseCoordinate((value.substr(start, value.size())).c_str(), 0, max_val);
	//LSLogPrintf(4, "xHotspot", "parse int: orig=%s ==> %d", value.c_str(), result);
	if(real_negative)
		return -result;
	else
		return result;

}

bool const util::string_to_bool(std::string const &value)
{
	return (to_lowercase(value) == "true") ? true : false;
}

HotspotType const util::string_to_type(std::string const &type)
{
	HotspotType result = Rectangular;
	if (to_lowercase(type) == "elliptic")
		result = Elliptic;
	return result;
}

StringList const util::GetNameList(std::string const &key)
// taken from the xlabel source (modified)
{
	StringList result;
	char line[MAX_LINE_LENGHT];
	FILE *fstep = LCOpen(0);

	while(LCReadNextConfig(fstep, key.c_str(), line, MAX_LINE_LENGTH))
	{
		char temp[MAX_CHAR] = "";
		char name[MAX_CHAR] = "";
		char *buffer[] = {temp, name};
			
		if(LCTokenize(line, buffer, 2, 0) >= 2)
		{
			result.insert(result.end(), name);
		}
	}
	LCClose(fstep);

	return result;
}

std::string const util::GetSetting(std::string const &name,
							 std::string const &setting,
							 std::string const &default_value,
							 bool const full_line)
{
	char buf[MAX_LINE_LENGHT];
	std::string result;

	BOOL found = 0;

	// check for individual setting
	if (full_line)
	{
		found = GetRCLine((name + setting).c_str(), buf, MAX_LINE_LENGHT, "");
		//LSLogPrintf(4, "xHotspot", "GetSetting reading: %s", buf);

		// Check if setting string contains surrounding quotation marks.
		// If it does, remove them by using tokenized string (which turns
		// "!bang arg1 arg2" into !bang arg1 arg2 (without quot. marks))
		// Why are we doing this?? Because LS wants bangs to start with !,
		// but if the bang is enclosed in quotation marks "!...", then it
		// doesn't work.
		int nToken = LCTokenize(buf, 0, 0, 0);
		LSLogPrintf(4, "xHotspot", "tokens: %d", nToken);
		if (nToken == 1)
		{
			// only one token found means that the command could be enclosed
			// in quotation marks. Therefore copy the quotation mark free version
			// back into buf
			char temp[MAX_LINE_LENGTH];
			char *temp_arr[] = {temp};
			LCTokenize(buf, temp_arr, 1, 0);
			//LSLogPrintf(4, "xHotspot", "buf: %s", buf);
			//LSLogPrintf(4, "xHotspot", "temp: %s", temp);
			StringCchCopy(buf, MAX_LINE_LENGTH, temp);
		}

	}
	else
		found = GetRCString((name + setting).c_str(), buf, "", MAX_LINE_LENGTH);
	if (found)
	{
		result = buf;
		if(result == "")  // setting found, but no value given
			// handle bool evars specially
			if (default_value == "false" || default_value == "true")
				result = "true";
			else
				result = default_value;
	}
	else // check for group membership and group setting
	{
		char group[MAX_CHAR];
		if (GetRCString((name + "AddToGroup").c_str(), group, "", MAX_CHAR))
		{
			std::string group_str(group);
			result = GetSetting(group_str, setting, default_value, full_line);
		}
		else result = default_value;
	}

	return result;
}

bool const util::Find(std::string const &exp, StringList const &strlist)
{
	for(StringList::const_iterator i = strlist.begin(); i != strlist.end(); ++i)
		if(*i == exp)
			return true;
	return false;
}