/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#if !defined (UTIL_H)
#define UTIL_H

#include "CustomTypes.h"

namespace util {

	/*
	 *  Guess what...? ;)
	 */
	std::string const to_lowercase(std::string const &str);

	/*
	 *  Converts a string to an int, without doing any Litestep number
	 *  evaluation.
	 */
	int const string_to_int(std::string const &value);

	/*
	 *  Turns a char[] containing a Litestep number to an int, parsing and
	 *  evaluating percent, negative and real negative values.
	 */
	int const parse_char_to_int(char const value[], int const &max_val);

	// Same as parse_char_to_int()
	int const parse_string_to_int(std::string const &value, int const &max_val);

	/*
	 *  Returns true if the string equals "true" (any case), else false.
	 */
	bool const string_to_bool(std::string const &value);

	HotspotType const string_to_type(std::string const &type);

	/*
	 *  Returns a list of names whose key matches the passed key
	 *  (for example all "*xHotspot"s)
	 */
	StringList const GetNameList(std::string const &key);

	/*
	 *  Reads a setting from step.rc, firstly checking for an individual setting.
	 *  In case none is found, it then checks for a group setting. If that isn't
	 *  found either, it returns the default value.
	 *  Set full_line to true in order to obtain the whole line and not just 
	 */
	std::string const GetSetting(std::string const &name,
								 std::string const &setting,
								 std::string const &default_value,
								 bool const full_line);

	/*
	 *  Looks for exp in strlist and returns true if it is found, false else.
	 */
	bool const Find(std::string const &exp, StringList const &strlist);
}

#endif