/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "xHotspot.h"

#include <strsafe.h>

#include "stdafx.h"
#include "Version.h"

#define LS_MESSAGE_LIST	LM_GETREVID, LM_REFRESH, 0

// the instance of our main class
xHotspot *xhotspot;

extern int monitor_width;
extern int monitor_height;


int initModuleEx(HWND hParent, HINSTANCE dllInst, LPCSTR szPath)
{
	//LSLogPrintf(4, "xHotspot", "===========================================", 0);
	hInst = dllInst;

	// create main window class (message handler)
	WNDCLASSEX wc = {0};

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = szAppName;
	wc.hIconSm = 0;
	
	if(!RegisterClassEx(&wc))
	{
		MessageBox(0, "xHotspot failed to register message handler class", "Error", MB_OK);
		return -1;
	}	

	// create window class for timer callbacks
	wc.lpfnWndProc = HotspotManager::WindowProc;
	wc.lpszClassName = "xHotspotManager";
	if(!RegisterClassEx(&wc))
	{
		MessageBox(0, "xHotspot failed to register manager class", "Error", MB_OK);
		return -1;
	}	

	// create main window
	hwnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		szAppName, //CLASSNAME,
		0,		// Window Name
		WS_POPUP,		// Window style
		0, 0, 0, 0,		// zero-sized  
		0/*HWND_MESSAGE*/,	// message-only window
		0,				// hMenu
		hInst,		// hInstance
		0				// lpParam
	);

	if (!hwnd)
	{
		/*
		void* pMsgBuf = NULL;  
	    FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER |
			FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,GetLastError(),0,(LPSTR)&pMsgBuf,0,NULL);
		std::string sReturn((LPTSTR)pMsgBuf);
    	LocalFree(pMsgBuf);
		*/
		MessageBox(0, "xHotspot failed to create main window.", "Error", MB_OK);
		return -1;
	}

	// exclude ourselves from vwm
	SetWindowLong(hwnd, GWL_USERDATA, magicDWord);

	int lsMessages[] = { LS_MESSAGE_LIST };

	SendMessage(GetLitestepWnd(),
				LM_REGISTERMESSAGE,
				(WPARAM) hwnd,
				(LPARAM) lsMessages);

	AddBangCommand("!xHotspotCreate", BangxHotspotCreate);
	AddBangCommand("!xHotspotDestroy", BangxHotspotDestroy);
	AddBangCommand("!xHotspotSetType", BangxHotspotSetType);
	AddBangCommand("!xHotspotEnable", BangxHotspotEnable);
	AddBangCommand("!xHotspotDisable", BangxHotspotDisable);
	AddBangCommand("!xHotspotToggle", BangxHotspotToggle);
	AddBangCommand("!xHotspotMove", BangxHotspotMove);
	AddBangCommand("!xHotspotMoveBy", BangxHotspotMoveBy);
	AddBangCommand("!xHotspotResize", BangxHotspotResize);
	AddBangCommand("!xHotspotResizeBy", BangxHotspotResizeBy);
	AddBangCommand("!xHotspotReposition", BangxHotspotReposition);
	AddBangCommand("!xHotspotRepositionBy", BangxHotspotRepositionBy);
	AddBangCommand("!xHotspotIfInside", BangxHotspotIfInside);

	// be sure to get monitor size before LoadHotspots()
	monitor_width = GetSystemMetrics(SM_CXSCREEN);
	monitor_height = GetSystemMetrics(SM_CYSCREEN);

	xhotspot = new xHotspot();
	xhotspot->LoadHotspots();

	return 0;
}

void quitModule(HINSTANCE dllInst) {

	int lsMessages[] = { LS_MESSAGE_LIST };

	SendMessage(GetLitestepWnd(),
				LM_UNREGISTERMESSAGE,
				(WPARAM) hwnd,
				(LPARAM) lsMessages);

	RemoveBangCommand("!xHotspotCreate");
	RemoveBangCommand("!xHotspotDestroy");
	RemoveBangCommand("!xHotspotSetType");
	RemoveBangCommand("!xHotspotEnable");
	RemoveBangCommand("!xHotspotDisable");
	RemoveBangCommand("!xHotspotToggle");
	RemoveBangCommand("!xHotspotMove");
	RemoveBangCommand("!xHotspotMoveBy");
	RemoveBangCommand("!xHotspotResize");
	RemoveBangCommand("!xHotspotResizeBy");
	RemoveBangCommand("!xHotspotReposition");
	RemoveBangCommand("!xHotspotRepositionBy");
	RemoveBangCommand("!xHotspotIsInside");

	DestroyWindow(hwnd);
	delete xhotspot;

	UnregisterClass(szAppName, dllInst);
	UnregisterClass("xHotspotManager", dllInst);
}


// I don't know, every module seems to have this... so here it is
BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);

	return TRUE;
}


LRESULT CALLBACK MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//LSLogPrintf(4, "xHotspot", "MessageHandlerProc", 0);

	LPSTR buffer = (LPSTR)lParam;

	switch(message)
	{
		case WM_CREATE:
			break;

		case LM_GETREVID:
			switch(wParam)
			{
				case 0:
				case 1:
				{
					LPSTR buf = (LPSTR)lParam;
					char module_info[64];
					StringCchCopy(module_info, 64, szModuleName);
					StringCchCat(module_info, 64, ": ");
					StringCchCat(module_info, 64, szModuleVersion);
					StringCchCopy(buf, 64, module_info);
					return strlen(buf);
					break;
				}
				default:
					StringCchCopy(buffer, 64, "");
					break;

			}

		case LM_REFRESH:
			xhotspot->LoadHotspots();
			break;

	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}
